// Begin include libraries
#include <BobShield.h> // Custom library for Bob Shield v3
#include <Commands.h>
#include <GPGGA.h>

/* In "/libraries/Wire/utility/twi.h" make sure TWI_FREQ (line 28) is defined as 400000L */
#include <Wire.h> // I2C communications (supporting the LSM303 library)
#include <LSM303.h> // Custom library for the compass board
#include <L3G.h> // Custom library for the L3GD20 gyro

#include <Flash.h> // Custom library for the NAND Flash
#include <SPI.h> // SPI communications (supporting the NAND-Flash library)
#include <EEPROM.h> // For non-volatile memory on the Arduino (supporting the NAND-Flash library)
// End include libraries

/* Try not to send data at a rate greater than ~30kbps for each Bob */
#define SAMPLE_INTERVAL (10000) // 100 Hz
#define PIP_IN A0

Flash* nandFlash; // Pointer to Flash object
BobShield* bobShield; // Pointer to BobShield object
LSM303 compass; // Compass object
L3G gyro; // Gyro object

// At the start of each line of data
byte DATA_SENTINEL = '#'; // To indicate a line of compass data

void setup() {
  Serial.begin(115200);
  initSensors();

//  nandFlash = new Flash(true, false); // Flash(enable_write, restart_address_counter)
  bobShield = new BobShield();
  delay(500);
  bobShield->flushBufferSPI();
  delay(500);
  bobShield->dntReset();
}

void loop() {
  // 3 acc, 3 mag, 3 gyro
  int data[10];
  unsigned long time = 0; // Time of last/current sample
  
  while(1)
  { 
    // Control the sampling cadence
    if ((micros() - time) > SAMPLE_INTERVAL)
    {
      // Collect data as close to time stamp as possible
      time = micros();
      compass.read();
      gyro.read();
      data[0] = (int)compass.a.x;
      data[1] = (int)compass.a.y;
      data[2] = (int)compass.a.z;
      data[3] = (int)compass.m.x;
      data[4] = (int)compass.m.y;
      data[5] = (int)compass.m.z;
      data[6] = (int)gyro.g.x;
      data[7] = (int)gyro.g.y;
      data[8] = (int)gyro.g.z;
      data[9] = (unsigned int)analogRead(PIP_IN);
  
      // Send data over the radio
      bobShield->dntSendData((uint8_t *)(&DATA_SENTINEL),1);
      bobShield->dntSendData((uint8_t *)(&time),4);
      bobShield->dntSendData((uint8_t *)data,20);
      
//      // Send data over the USB
//      Serial.write(DATA_SENTINEL);
//      Serial.write((byte *)(&time),4);
//      Serial.write((byte *)data,20);
      
//      // Send it to the nandflash
//      nandFlash->writeBytes(&DATA_SENTINEL,1);
//      nandFlash->writeBytes((byte *)(&time),4);
//      nandFlash->writeBytes((byte *)data,20);
    }
  }
}

// ---------------------------- Sensor Setup ----------------------------- //
void initSensors() {
  // Compass setup
  Wire.begin();
  compass.init(); //Turns on LSM303DLH
  compass.writeAccReg(LSM303_CTRL_REG1_A, 0x57); // Acc output data rate (ODR) = 100 Hz, not low power 
  compass.writeAccReg(LSM303_CTRL_REG4_A, 0x38); // Set accelerometer scale to +/-16G w HR
  compass.writeMagReg(LSM303_MR_REG_M, 0x00);    // Default
  compass.writeMagReg(LSM303_CRA_REG_M, 0x18);   // Mag output data rate (ODR) = 75 Hz
  compass.writeMagReg(LSM303_CRB_REG_M, 0x20);   // Mag gain +/-1.3 Gauss (X,Y - 1100 LSB/Gauss, Z - 980 LSB/Gauss)

  gyro.init(); //Turns on L3GD20
  gyro.writeReg(L3G_CTRL_REG1, 0x0F); // ODR 95Hz, high-pass filter cutoff 12.5, normal mode, all axes
  gyro.writeReg(L3G_CTRL_REG2, 0x00); // Normal mode hp-filter, 7.2 Hz cutoff @ 95Hz ODR
  gyro.writeReg(L3G_CTRL_REG4, 0x30); // Continuous update, little endian, scale 2000 dps
  
  // adjust the prescale for the analog inputs (http://arduino.cc/forum/index.php/topic,6549.0.html)
  // defines for setting and clearing register bit
  #ifndef cbi
  #define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
  #endif
  #ifndef sbi
  #define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
  #endif
  
  // set prescale to 32
  sbi(ADCSRA,ADPS2) ;
  cbi(ADCSRA,ADPS1) ;
  sbi(ADCSRA,ADPS0) ;
  
  pinMode(PIP_IN,INPUT);
}

