#include "BobShield.h"
#include <SPI.h>
#include <avr/io.h>

//#define DEBUG
	
#define SLAVE_SELECT_PIN 7
		
#define delayClockCycles(cycles) {for (int _cycleCounter = 0; _cycleCounter < (cycles); _cycleCounter++) {__asm__("nop\n\t");}}
		
// Constructors ////////////////////////////////////////////////////////////////

BobShield::BobShield()
{
	// Set up SPI
	SPI.begin(); 
	SPI.setDataMode(SPI_MODE0);
	SPI.setClockDivider(SPI_CLOCK_DIV2);
	this->slaveSelect_bm = (1 << SLAVE_SELECT_PIN);
	pinMode(SLAVE_SELECT_PIN, OUTPUT);
	this->csh();
}

// Public Methods //////////////////////////////////////////////////////////////

uint8_t BobShield::addOne(uint8_t data)
{
	this->csl();
	
	SPI.transfer(SlaveCommand_SpeedTest);	
	
	delayClockCycles(10);
	
	SPI.transfer(data);
	
	delayClockCycles(10);
	
	uint8_t response = SPI.transfer(SlaveCommand_Dummy);
	
	this->csh();
	
	return response;
}

/*
// status
//
// Gets the status of the bobshield
// Format: 000000(GPS_Data_Available)(DNT_Data_available)
*/
uint8_t BobShield::status()
{
	this->csl();
			
	SPI.transfer(SlaveCommand_Status);
			
	delayClockCycles(10);
				
  uint8_t s = SPI.transfer(SlaveCommand_Dummy);
			
	this->csh();
	
	return s;
}

void BobShield::setBaudDNT(BaudOptions baud)
{
	this->csl();
	
	SPI.transfer(SlaveCommand_ChangeBaud & baud);
	
	this->csh();
}

void BobShield::flushBufferSPI()
{
	this->csl();
	
	for (int i = 0; i < 10; i++)
	{
		SPI.transfer(SlaveCommand_Dummy);
		delayClockCycles(50);
	}
	
	this->csh();
}

bool BobShield::getAck(int numTries)
{
	this->csl();
	
	for (int i = 0; i < numTries; i++)
	{
		SPI.transfer(SlaveCommand_ACK);
		delayClockCycles(10);
	  if (SPI.transfer(SlaveCommand_Dummy) == SLAVE_ACK)
		{
			this->csh();
			return true;
		}
		delayClockCycles(10);
	}
	
	this->csl();
	return false;
}

void BobShield::readADC(bool ch0, bool ch1, bool toDNT, bool toMaster, uint8_t* ch0Data, uint8_t* ch1Data)
{
	this->csl();
	
	uint8_t command = SlaveCommand_ReadADC | (ch0 * PipSelect_0) | (ch1 * PipSelect_1) | (toDNT * Data_ToDNT) | (toMaster * Data_ToMaster); 
	
	SPI.transfer(command);
		
	if (toMaster)
	{
		if (ch0 && ch1)
		{
			delayClockCycles(25);
			*ch0Data = SPI.transfer(SlaveCommand_Dummy);
			delayClockCycles(25);
			*ch1Data = SPI.transfer(SlaveCommand_Dummy);
		}
		else if (ch0)
		{
			delayClockCycles(25);
			*ch0Data = SPI.transfer(SlaveCommand_Dummy);
		}
		else if (ch1)
		{
			delayClockCycles(25);
			*ch1Data = SPI.transfer(SlaveCommand_Dummy);
		}
	}
	this->csh();
}

void BobShield::writeDAC(uint16_t data, bool ch0, bool ch1)
{
	this->csl();
		
	SPI.transfer(SlaveCommand_WriteDAC | (PipSelect_0) | (PipSelect_1));
	delayClockCycles(10);
	
	SPI.transfer((data >> 8) & 0xFF);
	delayClockCycles(10);
	
	SPI.transfer(data & 0xFF);
	
	this->csh();
}

/*
// sweep
//
// Sweeps the pips.
// Currently unimplemented
*/
uint8_t* BobShield::sweep(uint16_t numSamples, bool sendToDNT, bool sendBack)
{
	this->csl();
	
	this->csh();
}

/*
// dntBytesAvailable
//
// Returns the number of bytes in the DNT
// receive buffer. Maximum of 255.
*/
uint8_t BobShield::dntBytesAvailable()
{
	this->csl();
	
  SPI.transfer(SlaveCommand_DNTNumAvailable);
	
	delayClockCycles(30);

	uint8_t numBytes = SPI.transfer(SlaveCommand_Dummy);
	
	this->csh();
	
	return numBytes;
}

/*
// dntSendData
//
// Sends length bytes of data over the DNT.
// Length has a maximum of 255.
*/
void BobShield::dntSendData(uint8_t* data, uint8_t length)
{
	this->csl();
	
  SPI.transfer(SlaveCommand_SendToDNT);
	delayClockCycles(10);
	
	SPI.transfer(length);
	
	for (int i = 0; i < length; i++)
	{
		delayClockCycles(300);
		SPI.transfer(data[i]);
	}

	this->csh();
}

/*
// dntReceiveData
// 
// Attempts to get max bytes of data from the DNT's receive buffer
// Returns the number of data points actually retrieved. 
// Maximum of 255 bytes.
*/
uint8_t BobShield::dntReceiveData(uint8_t* data, uint8_t max)
{
	this->csl();
	
  	SPI.transfer(SlaveCommand_DNTGet);
	
	delayClockCycles(10);
	
	SPI.transfer(max);
	
	delayClockCycles(10);
	
	uint8_t numBytes = SPI.transfer(SlaveCommand_Dummy);
			
	if (numBytes >= max)
	{
		this->flushBufferSPI();
		this->csh();
		return 0;
	}
	
	for (int i = 0; i < numBytes; i++)
	{
		delayClockCycles(100);
		data[i] = SPI.transfer(SlaveCommand_Dummy);
	}
							
	this->csh();
				
	return numBytes;
}

/*
// dntReset
//
// Resets the dnt radio
*/
void BobShield::dntReset()
{
	this->csl();
	
	SPI.transfer(SlaveCommand_DNTReset);
	
	delay(7000);
	
	this->csh();	
}

/*
// gpsGetData
//
// Gets the GPS data from the shield
// Unimplemented
*/
void BobShield::gpsSend()
{
	this->csl();
	
	SPI.transfer(SlaveCommand_GPSGet | Data_ToDNT);
	
	this->csh();	
}

uint8_t BobShield::gpsGet(uint8_t* data)
{
	this->csl();
	
	SPI.transfer(SlaveCommand_GPSGet | Data_ToMaster);
		
	delayClockCycles(20);
	
	int i = 0;		
	while (1)
	{
		delayClockCycles(150);
		data[i] = SPI.transfer(SlaveCommand_Dummy);
		
		if (!data[i])
		{
			break;
		}
		
		i++;
	}
							
	this->csh();
	
	return i;
}

uint8_t BobShield::gpsSendGet(uint8_t* data)
{
	this->csl();
	
	SPI.transfer(SlaveCommand_GPSGet | Data_ToMaster | Data_ToDNT);
		
	delayClockCycles(20);
	
	int i = 0;		
	while (1)
	{
		delayClockCycles(200);
		data[i] = SPI.transfer(SlaveCommand_Dummy);
		
		if (!data[i])
		{
			break;
		}
		
		i++;
	}
							
	this->csh();
	
	return i;
}

// Bring the chip select pin high
void BobShield::csh()
{
  PORTD |= this->slaveSelect_bm;
}

/// Bring the chip select pin low
void BobShield::csl()
{
  PORTD &= !this->slaveSelect_bm;
}
