/*
 * File: parser.cpp
 *
 * Written by Peter Horak in 2012 at Dartmouth College for the GreenCube project,
 * supervised by Professor Kristina Lynch.
 *
 * Purpose: parse binary sensor and ascii GPS data from the DaveBob payload
 */

#include <iostream>
#include <iomanip>
#include <string>
#include <sstream>
#include <fstream>

#include <stdint.h>
#include <stdio.h>
#include <math.h>

// Uncommented: split gps and compass board data and store in separate files
// Commented out: all data goest to stdout (cout)
#define FILE_OUT
#define NAND

using namespace std;

// Used to find the start of each data group
const char data_sentinel = '#';
#ifdef NAND
    const char gps_sentinel = '$';
#else
    const char gps_sentinel = 'G'; 
#endif

int main(int argc, char **argv)
{
    // To hold the data
    uint32_t time; // 4 bytes (unsigned int)
    uint32_t press; // 4 bytes (unsigned int)
    int16_t data[12]; // 2 bytes each (short)
    char c; // 1 byte
    
#ifdef FILE_OUT
    fstream gps_file; // Output file for parsed GPS data
    fstream cmp_file; // Output fiel for the parsed compass data
    
    gps_file.open("./gps_file.txt",fstream::out);
    cmp_file.open("./data_file.txt",fstream::out);
#else
    ostream & gps_file = cout;
    ostream & cmp_file = cout;
#endif
    
    // Convert the data into text and output it to the data buffer
    do {
        // Find the start of a line
        while ((c != data_sentinel) && (c != gps_sentinel))
            cin.read((&c),sizeof(c));
        // Parse a data line
        if (c == data_sentinel)
        {
            // micros() timestamp
            cin.read(reinterpret_cast<char *>(&time),sizeof(time));
            cmp_file << time << ' ';
            
            // Analog and compass data
            for (int i = 0; i < sizeof(data)/sizeof(data[0]); i++)
            {
                cin.read(reinterpret_cast<char *>(&data[i]),sizeof(data[i]));
                cmp_file << setw(5) << data[i] << ' ';
            }
            cin.read(reinterpret_cast<char *>(&press),sizeof(press));
            cmp_file << press;
            
            // Update c so it is no longer the previous sentinel
            cin.read((&c),sizeof(c));
            cmp_file << endl;
        }
        else // Handle a GPS string
        {
            // micros() timestamp
#ifdef NAND
            cin.read(reinterpret_cast<char *>(&time),sizeof(time));
            cin.read((&c),sizeof(c)); // WHY?
#endif
            
            // To temporarily store the GPS tring
            stringstream gps_buf;
            
            cin.read((&c),sizeof(c));
            while (c != data_sentinel)
            {
                gps_buf << c;
                cin.read((&c),sizeof(c));
            }
            
            /* Code to parse GPS data */
            // Values to read fromt the GPS string
            float timef;
            float lat; char a1[2];
            float lng; char a2[2];
            int stat; int nSat;
            float hor; float altf=0;
            
#ifdef NAND
            sscanf(gps_buf.str().c_str(),"GPGGA,%f,%f,%1s,%f,%1s,%d,%d,%f,%f", &timef, &lat, a1, &lng, a2, &stat, &nSat, &hor, &altf);
#else
            sscanf(gps_buf.str().c_str(),"PGGA,%f,%f,%1s,%f,%1s,%d,%d,%f,%f", &timef, &lat, a1, &lng, a2, &stat, &nSat, &hor, &altf);
#endif
            
            // Convert the time to hh:mm:ss format
            int timeg = round(timef);
            int hr = timeg/10000;
            int mi = timeg/100-hr*100;
            int sc = timeg-hr*10000-mi*100;
            
            // Get GPS coordinates entirely in degrees
            int deg = floor(lat/100);
            float min = lat - deg*100;
            lat = deg + min/60.0;
            
            deg = floor(lng/100);
            min = lng - deg*100;
            lng = -1.0*(deg+min/60.0); // -1 really comes from a1 and a2, but we don't need to check because we're always in New England so it doesn't change
            
            int alt = round(altf);
            
            // if (alt > 0)  // origional line
			if(1)  //rc 9/25/2014
            {
                gps_file << time << " " << setfill('0') << setw(2) << hr << " " << setw(2) << mi << " " << setw(2) << sc << "\t" << lat << " " << lng << " " << alt << endl;
            }
            /* end GPS parsing code */
        }
    } while (cin && !cin.eof());

#ifdef FILE_OUT
    gps_file.close();
    cmp_file.close();
#endif
}

