/*
 * File: preparser.cpp
 *
 * Written by Peter Horak in 2013 at Dartmouth College for the GreenCube project,
 * supervised by Professor Kristina Lynch.
 *
 * Purpose: parse DNT protocol messages from Bob Main
 */


#include <iostream>
#include <iomanip>
#include <string>
#include <sstream>
#include <fstream>
#include <map>

using namespace std;

void getEvent(void);
void rx_data(int bytes);
void announcement(int bytes);
void print_hex(int bytes);

int main(int argc, char **argv)
{
    cout << setfill('0');
    while (true) getEvent();
    
//    map<int,fstream>::iterator it;
//    for(it = files.begin(); it != files.end(); it++)
//        (it->second).close();
}

// Detect an event and print an appropriate message
void getEvent(void) {
    char c = '\0';
    
    // Find start-of-message byte
    while (c != (char)0xFB)
    {
        if (cin.eof()) return;
        cin.read((&c),sizeof(c));
    }
    
    // Get message length (bytes)
    if (cin.eof()) return; cin.read((&c),sizeof(c));
    int bytes = (unsigned char)c;
    
    // Get message type
    if (cin.eof()) return; cin.read((&c),sizeof(c));
    --bytes;
    
    switch (c) {
        case ((char)0x26): // RxData
            rx_data(bytes);
            break;
        case ((char)0x27): // Announce
            announcement(bytes);
            break;
        default: // Other event
            print_hex(bytes);
            cout << endl;
            break;
    }
}

void rx_data(int bytes) {
    // Associate one file with each observed mac addresss
    static map<unsigned int,ofstream *>files;

    // Expect at least 5 bytes (3 mac, 1 rssi, data)
    if (bytes < 5) {
        print_hex(bytes);
        return;
    }
    
    // Read RxData packet info
    unsigned int pkt_info; // 4 bytes
    if (cin.eof()) return;
    cin.read(reinterpret_cast<char *>(&pkt_info),sizeof(pkt_info));
    bytes-=sizeof(pkt_info); // Remaining bytes
    
    unsigned int mac_addr = pkt_info & 0xFFFFFF; // 3 bytes (little endian)
    char rssi = ((pkt_info >> 24) & 0xFF); // 1 byte (2s complement)
    cout << hex << setw(6) << mac_addr << "(" << dec << bytes << "): " << (int)rssi << endl;
    
    // Read in the data (all the remaining bytes)
    char data[bytes];
    if (cin.eof()) return;
    cin.read(data,sizeof(data));
    
    // If a file has not been opened for this mac address, open one now
    if (!files[mac_addr]) {
        stringstream ss;
        ss << hex << setw(6) << setfill('0') << mac_addr << ".dat";
        files[mac_addr] = new ofstream(ss.str().c_str(),ofstream::out);
    }
    
    // Write the data to a file
    files[mac_addr]->write(data,sizeof(data));
    files[mac_addr]->flush();
}

void announcement(int bytes) {
    int count = 0;
    if (bytes <= count) return;
    
    // Get status code
    char c = '\0';
    if (cin.eof()) return;
    cin.read((&c),sizeof(c));
    ++count;
    
    // Print the messages corresponding to the status code
    switch (c) {
        case ((char)0xA0):
            cout << "Radio has completed startup initialization.";
            break;
        case ((char)0xA1): // Custom announcement
            cout << "Radio has not recieved data recently.";
            break;
        case ((char)0xA2):
            cout << "Base: a child has joined the network.";
            break;
        case ((char)0xA3):
            cout <<"Remote: joined a network, ready for data.";
            break;
        case ((char)0xA4):
            cout << "Remote: exited network.";
            break;
        case ((char)0xA5):
            cout << "Remote: the base has rebooted.";
            break;
        case ((char)0xA7):
            cout << "Base: remote has left the network.";
            break;
        case ((char)0xA8):
            cout << "Base: heartbeat received from remote.";
            break;
        case ((char)0xA9):
            cout << "Base: router heartbeat timeout.";
            break;
        default:
            cout << "Status code " << hex << setw(2) << (int)(unsigned char)c << " unknown.";
    }
    
    // Print any additional fields in hex
    print_hex(bytes-count);
    cout << endl;
}

// Print the protocol messages as hex
void print_hex(int bytes) {
    char c = '\0';
    
    while (bytes > 0) {
        if (cin.eof()) return;
        cin.read((&c),sizeof(c));
        //cout << hex << setw(2) << (int)(unsigned char)c << ' ';
        --bytes;
    }
}
