// Read serial and print it to stdout

#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* for memset etc */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <stdlib.h>  /* for atoi */

int main(int argc, char *argv[]){

    if (argc < 2)
    {
        fprintf(stderr, "Must specify a device.\n");
        return 1;
    }
    
    //char * fullPort = "/dev/tty.usbmodemfa131";
    char *fullPort = argv[1];
    
    int fd;
    fd = open(fullPort,(O_RDWR | O_NOCTTY | O_NDELAY));
    if (fd < 0)
    {
        fprintf(stderr, "Failed to connect to device.\n");
        return 1;
    }
    speed_t baudrate = 115200; // Default
    if (argc == 3)
    {
        baudrate = atoi(argv[2]);
    }
    
    //init port==================
    struct termios options;
    memset(&options,0,sizeof(options));
    // Set the baud rate
    cfsetispeed(&options, baudrate);
    cfsetospeed(&options, baudrate);
    // Enable the receiver and set local mode...
    options.c_cflag=CS8|CREAD|CLOCAL;
    options.c_cc[VMIN]=1;
    options.c_cc[VTIME]=5;
    // Set the new options for the port...
    tcsetattr(fd, TCSANOW, &options);
    //end init port==============
    
    char result[2];
    int iRawCh;
    fcntl(fd, F_SETFL, FNDELAY); // don't block serial read
    
    // Output file
//    FILE *fp;
//    fp = fopen("./output.txt","a");
//    if (fp == NULL)
//    {
//        fprintf(stderr, "Failed to open output file.\n");
//        return 1;
//    }
    
    while(1){
        iRawCh = read(fd, result, 1);
        if (iRawCh>0)
        {
            fprintf(stdout, "%c", result[0]);
            fflush(stdout);
//            fprintf(fp,"%c",result[0]);
//            fflush(fp);
        }
        else
            usleep(5000);
    }
    close(fd);
//    fclose(fp);
    return 0; // success
}
